/* if you have the gssapi libraries */
#undef SHIBRESOLVER_HAVE_GSSAPI

/* if you have the GNU gssapi libraries */
#undef SHIBRESOLVER_HAVE_GSSGNU

/* if you have the Heimdal gssapi libraries */
#undef SHIBRESOLVER_HAVE_GSSHEIMDAL

/* if you have the MIT gssapi libraries */
#undef SHIBRESOLVER_HAVE_GSSMIT

/* if you have GSS naming extensions support */
#undef SHIBRESOLVER_HAVE_GSSAPI_NAMINGEXTS
